#include "Engine.h"

Engine::Engine(Window * _window)
	: window(_window)
{
	init();
}


Engine::~Engine(void)
{
}

void Engine::init(void) {
	counter = 0;
	scene = new Scene(Settings::getWidth(), Settings::getHeight(), window);
}

void Engine::startGame()
{
	isRunning = true;

	glCullFace(GL_BACK);
	glClearColor(0.0f, 0.f, 0.f, 1.0f);
	glViewport(0, 0, window->getFramebufferWidth(), window->getFramebufferHeight());
	//glEnable(GLsynctovblank
	glShadeModel(GL_SMOOTH);
	//glEnable(GL_TEXTURE_2D);
	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	//glfwWindowHint(GLFW_SAMPLES, 64);

	while(!window->shouldClose() && !scene->gameIsOver()) {
		glClearColor(1.0f, 1.f, 1.f, 1.0f);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
		update(window->getDeltaT());
				
		//draw Shadowmapping
		glViewport(0, 0, window->getFramebufferWidth()*4, window->getFramebufferHeight()*4);
		glEnable(GL_POLYGON_OFFSET_FILL);
		glPolygonOffset(0.0f, 0.0f);
		glCullFace(GL_FRONT);
		draw(SceneObject::RENDERTYPE::DEPTHSHADER);

		//draw Objekts and transparency objects
		glViewport(0, 0, window->getFramebufferWidth(), window->getFramebufferHeight());
		glClearColor(0.0f, 0.f, 0.f, 1.0f);
		glDisable(GL_POLYGON_OFFSET_FILL);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glCullFace(GL_BACK);
		draw(SceneObject::RENDERTYPE::BASIC);
		draw(SceneObject::RENDERTYPE::TRANSPARENCY);

		//Bloom Flame

		//draw the flame
		glViewport(0, 0, window->getFramebufferWidth()*0.25, window->getFramebufferHeight()*0.25);
		glClearColor(0.0f, 0.f, 0.f, 0.0f);
		draw(SceneObject::RENDERTYPE::FLAMESHADER);
		
		////draw Bloom
		glClearColor(0.0f, 0.f, 0.f, 0.0f);
		draw(SceneObject::RENDERTYPE::BLOOMSHADER);

		glViewport(0, 0, window->getFramebufferWidth(), window->getFramebufferHeight());
		
		draw(SceneObject::RENDERTYPE::HUDSHADER);


		//DataOutput;
		scene->writeInformation(window->getDeltaT());

		window->resizeWindow();
		window->swapBuffers();

		glfwPollEvents();

		GLenum error = glGetError();
 
		if (error != GL_NO_ERROR) {
			switch (error) {
				case GL_INVALID_ENUM:
					std::cerr << "GL: enum argument out of range." << std::endl;
					break;
				case GL_INVALID_VALUE:
					std::cerr << "GL: Numeric argument out of range." << std::endl;
					break;
				case GL_INVALID_OPERATION:
					std::cerr << "GL: Operation illegal in current state." << std::endl;
					break;
				case GL_INVALID_FRAMEBUFFER_OPERATION:
					std::cerr << "GL: Framebuffer object is not complete." << std::endl;
					break;
				case GL_OUT_OF_MEMORY:
					std::cerr << "GL: Not enough memory left to execute command." << std::endl;
					break;
				default:
					std::cerr << "GL: Unknown error." << std::endl;
			}
		}
	}

}

void Engine::update(GLdouble deltaT) 
{
	scene->update(window->getDeltaT()*(0.5+Settings::getDifficulty()/2.0));
}

void Engine::draw(SceneObject::RENDERTYPE mode) 
{
	//shader->useShader();
	scene->draw(mode);
}


