#include "ShaderManager.h"

ShaderManager* ShaderManager::_instance = 0;

ShaderManager::ShaderManager() 
{
	std::cout << "ShaderManager: initializing ..." << std::endl;
	shader = new Shader("ExFlame/Shader/basicShader.vert","ExFlame/Shader/basicShader.frag");
	depthShader = new Shader("ExFlame/Shader/depthShader.vert","ExFlame/Shader/depthShader.frag");
	flameShader = new Shader("ExFlame/Shader/flameShader.vert","ExFlame/Shader/flameShader.frag");
	hudShader = new Shader("ExFlame/Shader/hudShader.vert", "ExFlame/Shader/hudShader.frag");
	bloomShader = new Shader("ExFlame/Shader/glowShader.vert", "ExFlame/Shader/glowShader.frag");

	//BasicShader VertexShader
	listBasic["projectionMatrix"] = glGetUniformLocation(shader->programHandle,"projectionMatrix");
	listBasic["modelMatrix"] = glGetUniformLocation(shader->programHandle,"modelMatrix");
	listBasic["viewMatrix"] = glGetUniformLocation(shader->programHandle,"viewMatrix");
	listBasic["camPosition"] = glGetUniformLocation(shader->programHandle,"camPosition");
	listBasic["light"] = glGetUniformLocation(shader->programHandle,"light");
	listBasic["depthProjMat"] = glGetUniformLocation(shader->programHandle,"depthProjMat");
	listBasic["depthViewMat"] = glGetUniformLocation(shader->programHandle,"depthViewMat");
	listBasic["biasMat"] = glGetUniformLocation(shader->programHandle,"biasMat");
	listBasic["depthMVP"] = glGetUniformLocation(shader->programHandle,"depthMVP");

	//BasicShader FragmentShader
	listBasic["material_ambient"] = glGetUniformLocation(shader->programHandle,"material_ambient");
	listBasic["material_diffuse"] = glGetUniformLocation(shader->programHandle,"material_diffuse");
	listBasic["material_specular"] = glGetUniformLocation(shader->programHandle,"material_specular");
	listBasic["transparency"] = glGetUniformLocation(shader->programHandle,"transparency");
	listBasic["border"] = glGetUniformLocation(shader->programHandle,"border");
	listBasic["lightColor"] = glGetUniformLocation(shader->programHandle,"lightColor");
	listBasic["maxDistance"] = glGetUniformLocation(shader->programHandle,"maxDistance");
	listBasic["minDistance"] = glGetUniformLocation(shader->programHandle,"minDistance");
	listBasic["distance"] = glGetUniformLocation(shader->programHandle,"distance");
	listBasic["color_texture"] = glGetUniformLocation(shader->programHandle,"color_texture");
	listBasic["normal_texture"] = glGetUniformLocation(shader->programHandle,"normal_texture");
	listBasic["shadow_texture"] = glGetUniformLocation(shader->programHandle,"shadow_texture");

	//DepthShader VertexShader
	listDepth["modelMatrix"] = glGetUniformLocation(depthShader->programHandle,"modelMatrix");
	listDepth["projectionMatrix"] = glGetUniformLocation(depthShader->programHandle,"projectionMatrix");
	listDepth["viewMatrix"] = glGetUniformLocation(depthShader->programHandle,"viewMatrix");
	listDepth["camPosition"] = glGetUniformLocation(depthShader->programHandle,"camPosition");
	listDepth["light"] = glGetUniformLocation(depthShader->programHandle,"light");

	//FlameMatrix
	listFlame["modelMatrix"] =glGetUniformLocation(flameShader->programHandle,"modelMatrix");
	listFlame["projectionMatrix"] =glGetUniformLocation(flameShader->programHandle,"projectionMatrix");
	listFlame["viewMatrix"] =glGetUniformLocation(flameShader->programHandle,"viewMatrix");
	//FlameMatrix
	listFlame["lightColor"] =glGetUniformLocation(flameShader->programHandle,"lightColor");

	//BloomShader Vertex
	listBloom["modelMatrix"] =glGetUniformLocation(bloomShader->programHandle,"modelMatrix");
	//BloomShader Fragment
	listBloom["glowTexture"] =glGetUniformLocation(bloomShader->programHandle,"glowTexture");
	listBloom["windowHeight"] =glGetUniformLocation(bloomShader->programHandle,"windowHeight");
	listBloom["windowWidth"] =glGetUniformLocation(bloomShader->programHandle,"windowWidth");
	listBloom["pass"] =glGetUniformLocation(bloomShader->programHandle,"pass");
	listBloom["border"] = glGetUniformLocation(bloomShader->programHandle,"border");

	//HudShader
	listHud["modelMatrix"] = glGetUniformLocation(hudShader->programHandle,"modelMatrix");
	listHud["colorNeutral"] = glGetUniformLocation(hudShader->programHandle,"colorNeutral");
	listHud["colorFinal"] = glGetUniformLocation(hudShader->programHandle,"colorFinal");
	//listBasic[""] = glGetUniformLocation(shader->programHandle,"");
	std::cout << "ShaderManager: DONE" << std::endl;
}

GLint ShaderManager::getUniformLoc(SHADER s, std::string name) 
{
	GLint value;
	switch(s) {
	case SHADER::BASICSHADER:
		value = listBasic[name];
		break;
	case SHADER::DEPTHSHADER:
		value = listDepth[name];
		break;
	case SHADER::FLAMESHADER:
		value = listFlame[name];
		break;
	case SHADER::BLOOMSHADER:
		value = listBloom[name];
		break;
	case SHADER::HUDSHADER:
		value = listHud[name];
		break;
	default: break;
	}
	return value;
}

GLuint ShaderManager::getShaderProgramHandle(SHADER s) {

	GLuint value;
	switch(s) {
	case SHADER::BASICSHADER:
		value = shader->programHandle;
		break;
	case SHADER::DEPTHSHADER:
		value = depthShader->programHandle;
		break;
	case SHADER::FLAMESHADER:
		value = flameShader->programHandle;
		break;
	case SHADER::BLOOMSHADER:
		value = bloomShader->programHandle;
		break;
	case SHADER::HUDSHADER:
		value = hudShader->programHandle;
		break;
	default: break;
	}
	return value;
}

void ShaderManager::useShader(SHADER s) {

	switch(s) {
	case SHADER::BASICSHADER:
		shader->useShader();
		break;
	case SHADER::DEPTHSHADER:
		depthShader->useShader();
		break;
	case SHADER::FLAMESHADER:
		flameShader->useShader();
		break;
	case SHADER::BLOOMSHADER:
		bloomShader->useShader();
		break;
	case SHADER::HUDSHADER:
		hudShader->useShader();
		break;
	default: break;
	}
}

ShaderManager* ShaderManager::getInstance() {
	if(_instance == 0) _instance = new ShaderManager();

	return _instance;
}