#pragma once

#include <GL/glew.h>
#include <GLFW\glfw3.h>>
#include <glm\gtc\matrix_transform.hpp>
#include <GLM\glm.hpp>
#include <vector>

#include "Shader.h"
#include "Texture.h"
#include "SceneObject.h"

#include <ASSIMP\Importer.hpp>
#include <ASSIMP\postprocess.h>
#include <ASSIMP\scene.h>
#include <ASSIMP\cimport.h>

class ParticleSystem
	:SceneObject
		{
		public:
			ParticleSystem(glm::vec3 position, Shader* _shader, Shader* _depthShader, Shader* flameShader, Shader *particleShader);
			virtual ~ParticleSystem();

			virtual void update(double deltaT);
			virtual void draw(RENDERTYPE mode);
			virtual void updateObject(const char* pathToObj, Shader* _shader, Texture* _tex);

			virtual void loadOBJ(const char *path, std::vector<unsigned int> &outIndices, std::vector<float> &outVertices,
								std::vector<float> &outUVs, std::vector<float> &outNormals);

		private:
			int NumOfInstances;
			GLuint vao;
			GLuint positionBuffer, normalBuffer, indexBuffer, uvBuffer, modelMatsBuffer;
			Shader* shader, *depthShader, *flameShader, *particleShader;
			Texture* texture;

			std::vector<unsigned int> indices;
			std::vector<float> vertices;
			std::vector<float> uvs;
			std::vector<float> normals;
			std::vector<glm::vec3> positionParticles;
			std::vector<float> lifetime;
			std::vector<glm::vec3> color;
			std::vector<glm::fmat4x4> modelMats;
};