#include "SceneObject.h"
#include "../Settings.h"
SceneObject::SceneObject(std::string filename,glm::vec3& _position, float _scaleTexture, float _scaleObject) 
	: position(_position)
	, scaleTexture(_scaleTexture)
	, scaleObject(_scaleObject)
{
	shaderManager = ShaderManager::getInstance();

	texture = nullptr;
	normalTexture = nullptr;

	orientation = glm::vec3(1.0f);
	direction = glm::vec3(0.0f);
	visability = 1.0f;

	modelMatrixParent = glm::mat4(0.0f);

	std::string fullfilepathOBJ = "ExFlame/3DModels/" + filename + ".obj";
	std::string fullfilepathTex = "ExFlame/3DModels/" + filename + "_diff.jpg";
	std::string fullfilepathNormalTex = "ExFlame/3DModels/" + filename + "_norm.jpg";
	
	loadOBJ(fullfilepathOBJ.c_str(), indices, vertices, uvs, normals, tangents, bitangents);
	loadTexture(fullfilepathTex.c_str());
	loadNormalTexture(fullfilepathNormalTex.c_str());

	//Load data to buffer
	glGenBuffers(1, &positionBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
	glBufferData(GL_ARRAY_BUFFER, vertices.size() * sizeof(float),
		&vertices[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0); //unbind (mit 0)

		//Load data to buffer
	glGenBuffers(1, &normalBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
	glBufferData(GL_ARRAY_BUFFER, normals.size() * sizeof(float),
		&normals[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0); //unbind (mit 0)

	//Load data to buffer
	glGenBuffers(1, &uvBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
	glBufferData(GL_ARRAY_BUFFER, uvs.size() * sizeof(float),
		&uvs[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0); //unbind (mit 0)

	//Load data to buffer
	glGenBuffers(1, &tangentsBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, tangentsBuffer);
	glBufferData(GL_ARRAY_BUFFER, tangents.size() * sizeof(float),
		&tangents[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0); //unbind (mit 0)

	//Load data to buffer
	glGenBuffers(1, &bitangentsBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, bitangentsBuffer);
	glBufferData(GL_ARRAY_BUFFER, bitangents.size() * sizeof(float),
		&bitangents[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0); //unbind (mit 0)

	//Load data to buffer
	glGenBuffers(1, &indexBuffer);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.size() * sizeof(unsigned int),
		&indices[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0); // unbind (mit 0)

	//Generate VAO
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
	
	GLint positionIndex = glGetAttribLocation(shaderManager->getShaderProgramHandle(ShaderManager::SHADER::BASICSHADER), "position");
	glEnableVertexAttribArray(positionIndex);
	glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
	GLint normalIndex = glGetAttribLocation(shaderManager->getShaderProgramHandle(ShaderManager::SHADER::BASICSHADER), "normal");
	glEnableVertexAttribArray(normalIndex);
	glVertexAttribPointer(normalIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
	GLint uvIndex = glGetAttribLocation(shaderManager->getShaderProgramHandle(ShaderManager::SHADER::BASICSHADER), "uv");
	glEnableVertexAttribArray(uvIndex);
	glVertexAttribPointer(uvIndex, 2, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER, tangentsBuffer);
	GLint tangentsIndex = glGetAttribLocation(shaderManager->getShaderProgramHandle(ShaderManager::SHADER::BASICSHADER), "tangents");
	glEnableVertexAttribArray(tangentsIndex);
	glVertexAttribPointer(tangentsIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER, bitangentsBuffer);
	GLint bitangentsIndex = glGetAttribLocation(shaderManager->getShaderProgramHandle(ShaderManager::SHADER::BASICSHADER), "bitangents");
	glEnableVertexAttribArray(bitangentsIndex);
	glVertexAttribPointer(bitangentsIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);
	
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);

	glBindVertexArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

}


SceneObject::~SceneObject()
{
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &normalBuffer);
	glDeleteBuffers(1, &indexBuffer);
	glDeleteBuffers(1, &uvBuffer);
	glDeleteVertexArrays(1, &vao);
}

void SceneObject::addChild(SceneObject * child) 
{
	children.push_back(child);
}

void SceneObject::updateModelMatrixAtChildren(glm::mat4 modelMat)
{
	modelMatrixParent = modelMat;
}

void SceneObject::draw(RENDERTYPE mode)
{
	if(mode == RENDERTYPE::DEPTHSHADER) {
		//DepthShader
	} else if(mode == RENDERTYPE::BASIC) {
		//BasicShader
		texture->bind(0);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "color_texture"),0);

		normalTexture->bind(1);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "normal_texture"),1);

		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "material_ambient"),material->getAmbient());
		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "material_diffuse"),material->getDiffuse());
		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "material_specular"),material->getSpecular());
		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "transparency"), 1.0f);
	} else if(mode == RENDERTYPE::FLAMESHADER) {
	} else if(mode == RENDERTYPE::TRANSPARENCY) {
		texture->bind(0);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER,"color_texture"),0);

		normalTexture->bind(1);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER,"normal_texture"),1);

		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER,"material_ambient"),material->getAmbient());
		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER,"material_diffuse"),material->getDiffuse());
		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER,"material_specular"),material->getSpecular());
	}

	glBindVertexArray(vao);
	glDrawElements(GL_TRIANGLES, indices.size(), GL_UNSIGNED_INT, 0);
	glBindVertexArray(0);
}

void SceneObject::update(double deltaT)
{
}

void SceneObject::update(glm::mat4 modelMat, double deltaT)
{
}

void SceneObject::rotate(glm::vec3 rotation, float deltaT)
{
}

void SceneObject::move(glm::vec3 direction) 
{
}

void SceneObject::jump()
{
}

void SceneObject::generateMM()
{
}

glm::vec3 SceneObject::getPosition()
{
	return position;
}
void SceneObject::resize(double size)
{
}

void SceneObject::setPosition(glm::vec3 _position)
{
	position = _position;
}

void SceneObject::loadOBJ(const char *path, std::vector<unsigned int> &outIndices,
					std::vector<float> &outVertices, std::vector<float> &outUVs,
					std::vector<float> &outNormals, std::vector<float> &outTangents, 
					std::vector<float> &outBitangents) {

	//std::cout<< "Load Model: " << path << std::endl;
	Assimp::Importer importer;

	const aiScene* scene = importer.ReadFile(path,
								aiProcess_JoinIdenticalVertices|
								aiProcess_Triangulate|
								aiProcess_OptimizeMeshes| 
								aiProcess_ConvertToLeftHanded
								);

	if(scene==NULL) std::cout << importer.GetErrorString() << std::endl;

	aiMesh* mesh = scene->mMeshes[0];

	int numOfFaces = mesh->mNumFaces;
	int numOfIndices = numOfFaces * 3;

	outIndices.resize(numOfIndices);
	
	for (unsigned int i=0; i < mesh->mNumFaces; ++i)
	{
		const aiFace &face = mesh->mFaces[i];
		assert(face.mNumIndices == 3);
		outIndices[i*3 + 0] = face.mIndices[0];
		outIndices[i*3 + 1] = face.mIndices[1];
		outIndices[i*3 + 2] = face.mIndices[2];
	}

	int numOfVertices = mesh->mNumVertices;
	outVertices.resize(numOfVertices * 3);
	outNormals.resize(numOfVertices * 3);
	outUVs.resize(numOfVertices * 2);
	outBitangents.resize(numOfVertices * 3);
	outTangents.resize(numOfVertices * 3);
	
	for (unsigned int i=0; i < mesh->mNumVertices; ++i)
	{
		if(mesh->HasPositions())
		{
			outVertices[i*3 + 0] = mesh->mVertices[i].x;
			outVertices[i*3 + 1] = mesh->mVertices[i].y;
			outVertices[i*3 + 2] = mesh->mVertices[i].z;
		}

		if(mesh->HasNormals())
		{
			outNormals[i*3 + 0] = mesh->mNormals[i].x;
			outNormals[i*3 + 1] = mesh->mNormals[i].y;
			outNormals[i*3 + 2] = mesh->mNormals[i].z;
		}

		/*if(mesh->HasTangentsAndBitangents())
		{
			outTangents[i*3 +0] = mesh->mTangents[i].x;
			outTangents[i*3 +1] = mesh->mTangents[i].y;
			outTangents[i*3 +2] = mesh->mTangents[i].z;

			outBitangents[i*3 +0] = mesh->mBitangents[i].x;
			outBitangents[i*3 +1] = mesh->mBitangents[i].y;
			outBitangents[i*3 +2] = mesh->mBitangents[i].z;
		}*/

		if(mesh->HasTextureCoords(0))
		{
			outUVs[i*2 + 0] = mesh->mTextureCoords[0][i].x * scaleTexture;
			outUVs[i*2 + 1] = mesh->mTextureCoords[0][i].y * scaleTexture;
		}	
	}

	//calculate Tangents and Bitangents
	for(unsigned int i=0; i<outVertices.size();i++){
		//outVertices[i];
		glm::vec3 &v0 = glm::vec3(outVertices.at((i*3+0)%outVertices.size()), outVertices.at((i*3+1)%outVertices.size()), outVertices.at((i*3+2)%outVertices.size()));
		glm::vec3 &v1 = glm::vec3(outVertices.at(((i+1)*3+0)%outVertices.size()), outVertices.at(((i+1)*3+1)%outVertices.size()), outVertices.at(((i+1)*3+2)%outVertices.size()));
		glm::vec3 &v2 = glm::vec3(outVertices.at(((i+2)*3+0)%outVertices.size()), outVertices.at(((i+2)*3+1)%outVertices.size()), outVertices.at(((i+2)*3+2)%outVertices.size()));

		glm::vec2 & uv0 = glm::vec2(outUVs.at(((i+0)*2+0)%outUVs.size()),outUVs.at(((i+0)*2+1)%outUVs.size()));
		glm::vec2 & uv1 = glm::vec2(outUVs.at(((i+1)*2+0)%outUVs.size()),outUVs.at(((i+1)*2+1)%outUVs.size()));
		glm::vec2 & uv2 = glm::vec2(outUVs.at(((i+2)*2+0)%outUVs.size()),outUVs.at(((i+2)*2+1)%outUVs.size()));

		glm::vec3 deltaPos1 = v1 - v0;
		glm::vec3 deltaPos2 = v2 - v0;

		glm::vec2 deltaUV1 = uv1 - uv0;
		glm::vec2 deltaUV2 = uv2 - uv0;

		float r = 1.0f / (deltaUV1.x * deltaUV2.y - deltaUV1.y * deltaUV2.x);
		glm::vec3 tangent = (deltaPos1 * deltaUV2.y - deltaPos2 * deltaUV1.y)*r;
		glm::vec3 bitangent = (deltaPos2 * deltaUV1.x - deltaPos1 * deltaUV2.x)*r;

		outTangents.at(((i+0)*3+0)%outVertices.size()) += tangent.x;
		outTangents.at(((i+0)*3+1)%outVertices.size()) += tangent.y;
		outTangents.at(((i+0)*3+2)%outVertices.size()) += tangent.z;

		outTangents.at(((i+1)*3+0)%outVertices.size()) += tangent.x;
		outTangents.at(((i+1)*3+1)%outVertices.size()) += tangent.y;
		outTangents.at(((i+1)*3+2)%outVertices.size()) += tangent.z;

		outTangents.at(((i+2)*3+0)%outVertices.size()) += tangent.x;
		outTangents.at(((i+2)*3+1)%outVertices.size()) += tangent.y;
		outTangents.at(((i+2)*3+2)%outVertices.size()) += tangent.z;

		outBitangents.at(((i+0)*3+0)%outVertices.size()) += bitangent.x;
		outBitangents.at(((i+0)*3+1)%outVertices.size()) += bitangent.y;
		outBitangents.at(((i+0)*3+2)%outVertices.size()) += bitangent.z;

		outBitangents.at(((i+1)*3+0)%outVertices.size()) += bitangent.x;
		outBitangents.at(((i+1)*3+1)%outVertices.size()) += bitangent.y;
		outBitangents.at(((i+1)*3+2)%outVertices.size()) += bitangent.z;

		outBitangents.at(((i+2)*3+0)%outVertices.size()) += bitangent.x;
		outBitangents.at(((i+2)*3+1)%outVertices.size()) += bitangent.y;
		outBitangents.at(((i+2)*3+2)%outVertices.size()) += bitangent.z;
	}

	//loadTextureAndMaterial(scene);
}

void SceneObject::loadTexture(const char * path)
{
	texture = new Texture(path);
}

void SceneObject::loadNormalTexture(const char *path)
{
	normalTexture = new Texture(path);
}

void SceneObject::playSound(SoundManager::SOUND_COLLECTION entry, bool loop)
{
	SoundManager::playSound(entry, SoundManager::SOUND_SOURCE::HERO, loop);
}

void SceneObject::setTextureSampling(int mode)
{
	texture->setTextureSampling(mode);
}

void SceneObject::setMipMapping(int mode)
{
	texture->setMipMapping(mode);
}