#pragma once

#include <GL\glew.h>
#include <GL\gl.h>
#include <glm\gtc\matrix_transform.hpp>
#include <GLM\glm.hpp>
#include <vector>

#include <ASSIMP\Importer.hpp>
#include <ASSIMP\postprocess.h>
#include <ASSIMP\scene.h>
#include <ASSIMP\cimport.h>
#include <GLM/gtc/type_ptr.hpp>
#include <time.h>
#include <cstdlib>
#include <iostream>
#include <string>
#include "Shader.h"
#include "Texture.h"
#include "../GameEngine/SoundManager.h"
#include "Material.h"

#include "../GameEngine/ShaderManager.h"
//#include "../GameEngine/KeyboardManager.h"

class SceneObject
{
public:
	SceneObject(std::string filename, glm::vec3& _position, float scaleTexture, float scaleObject);
	virtual ~SceneObject();

	enum RENDERTYPE {
		DEPTHSHADER,
		FLAMESHADER,
		BLOOMSHADER,
		BASIC,
		TRANSPARENCY,
		HUDSHADER,
		PARTICLESHADER
	};

	virtual void update(double deltaT);
	virtual void update(glm::mat4 modelMat, double deltaT);
	virtual void draw(RENDERTYPE mode);
	virtual void rotate(glm::vec3 rotation, float degree);
	virtual void move(glm::vec3 direction);
	virtual void jump();
	virtual void resize(double size);
	virtual void generateMM();
	virtual void playSound(SoundManager::SOUND_COLLECTION entry, bool loop);
	glm::vec3 getPosition(void);
	void SceneObject::rotate(glm::vec3 rotation, double deltaT);
	void setPosition(glm::vec3 position);

	void addChild(SceneObject *child);
	void setTextureSampling(int mode);
	void setMipMapping(int mode);
	virtual void updateModelMatrixAtChildren(glm::mat4 modelMat);
		
protected:
	GLuint vao;
	GLuint positionBuffer, normalBuffer, indexBuffer, uvBuffer, tangentsBuffer, bitangentsBuffer;
	Texture *texture;
	Texture *normalTexture;
	Material *material;

	std::vector<unsigned int> indices;
	std::vector<float> vertices;
	std::vector<float> uvs;
	std::vector<float> normals;
	std::vector<float> tangents;
	std::vector<float> bitangents;

	std::vector<SceneObject *> children;
	glm::mat4 modelMatrix;
	glm::vec3 position;
	glm::vec3 orientation;
	glm::vec3 direction;
	glm::mat4 modelMatrixParent;

	glm::vec3 restRotation;
	glm::vec3 restMovement;

	float scaleTexture, scaleObject,  visability;

	bool hasMusic, hasSoundFX;
	
	ShaderManager *shaderManager;

private: 
	void SceneObject::loadOBJ(const char *path, std::vector<unsigned int> &outIndices, std::vector<float> &outVertices,
		std::vector<float> &outUVs, std::vector<float> &outNormals, std::vector<float> &outTangents, std::vector<float> &outBitangents);
	void loadTexture(const char *path);
	void loadNormalTexture(const char *path);
};