#include "Shader.h"

#include <iostream>
#include <fstream>

#include <GL\glew.h>
#include <GLFW\glfw3.h>

Shader::Shader(const std::string& vertexShader, const std::string& fragmentShader)
	: programHandle(0),
	  fragmentHandle(0),
	  vertexHandle(0)
{

	programHandle = glCreateProgram();
	
	if(programHandle == 0){
		std::cout << "Failed to create Shader Program" << std::endl;
		system("PAUSE");
		exit(-1);
	}

	loadShader(vertexShader, GL_VERTEX_SHADER, vertexHandle);
	loadShader(fragmentShader, GL_FRAGMENT_SHADER, fragmentHandle);

	link();
}


Shader::~Shader()
{
	glDeleteProgram(programHandle);
	glDeleteShader(vertexHandle);
	glDeleteShader(fragmentHandle);
}

void Shader::useShader() const {
	glUseProgram(programHandle);
}

void Shader::loadShader(const std::string& shader, GLenum shaderType, GLuint& handle){

	std::ifstream shaderfile(shader);

	if (shaderfile.good()) {
		std::string code = std::string(
			std::istreambuf_iterator<char>(shaderfile),
			std::istreambuf_iterator<char>());
		shaderfile.close();
		handle = glCreateShader(shaderType);

		if(handle == 0) {
			std::cout << "Didn't get valid Shaderfile" << std::endl;
			system("PAUSE");
			exit(-1);
		}

		//compile Shader
		const char* codePtr = code.c_str();
		glShaderSource(handle, 1, &codePtr, nullptr);
		glCompileShader(handle);

		//test if compiled correctly
		GLint succeded;
		glGetShaderiv(handle, GL_COMPILE_STATUS, &succeded);

		//if shader incorrect -> message out and end program
		if (succeded == GL_FALSE || !glIsShader(handle)){
			GLint logSize;
			glGetShaderiv(handle, GL_INFO_LOG_LENGTH, &logSize);
			auto message = new char[logSize];
 
			glGetShaderInfoLog(handle, logSize, NULL, message);
			std::cout << "Failed to compile shader" << std::endl;
			std::cout << message << std::endl;
			system("PAUSE");
			delete[] message;
			exit(-1);
		}

	}
	else {
		std::cout << "Failed to open file " << shader << std::endl;
		system("PAUSE");
		exit(-1);
	}
}

void Shader::link(){
	glAttachShader(programHandle, vertexHandle);
	glAttachShader(programHandle, fragmentHandle);

	glLinkProgram(programHandle);
}